﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Xml;
using Adam.Core;
using Adam.Core.DatabaseManagerLibrary;
using Adam.Core.Management;
using Adam.Core.Settings;
using Adam.Core.Tools;
using Adam.Tools.CommandLine;
using Adam.Tools.LogHandler;

namespace Adam.Fadel.CommandLine
{
    public class Program
    {
        private static Arguments _arguments;
        private static Application _app;

        private static void Main(string[] args)
        {
            try
            {
                ConsoleHelper.WriteCopyright();
                // Making sure that we use a proper event log source...
                EventLogListener.Source = Application.EventLogSource + " CommandLine";
                _arguments = Arguments.Parse(args);
                Command[] commands =
                {
                    new Command("Install", Install, ShowInstallHelp, true)
                };
                _arguments.ExecuteCommand(commands.ToArray(), DisplayHelp);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                LogManager.Write(LogSeverity.Error, "The specified operation failed:", ex);
                Environment.Exit(1);
            }
        }

        private static void Install(string command, Arguments args)
        {
            string registration = args.GetVariableValue("registration", null);
            string userName = args.GetVariableValue("userName", null);
            string password = args.GetVariableValue("password", null);

            using (_app = new Application())
            {
                if (_app.LogOn(registration, userName, password) == LogOnStatus.LoggedOn)
                {
                    //Registering the assembly
                    string path = Path.Combine(
                        Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location),
                        "Adam.Fadel.Web.dll");
                    var oHelper = new ProviderHelper(_app);
                    oHelper.RegisterAssembly(path, true);

                    //Deploying image and css additions
                    IList<StudioConfigEntry> configEntries = StudioConfig.Open().GetStudiosOfRegistration(_app.RegistrationName);
                    foreach (StudioConfigEntry entry in configEntries)
                    {
                        if (entry.StudioType == BuiltInStudioType.AssetStudio.ToString())
                        {
                            Assembly assembly = Assembly.LoadFrom(Path.Combine(entry.FileSystemPath, @"bin\Adam.Web.AssetStudio.dll"));
                            string defaultStyleFolder = Path.Combine("Style", assembly.GetName().Version.ToString().Replace(".", ""));

                            string logopath = Path.Combine(entry.FileSystemPath, defaultStyleFolder, @"Images\Fadel_Logo_s.png");
                            string customcsspath = Path.Combine(entry.FileSystemPath, defaultStyleFolder, "Custom.css");
                            if (!File.Exists(logopath))
                            {
                                ExtractResource("Adam.Fadel.CommandLine.Fadel_Logo_s.png", logopath);
                            }

                            //Adding the custom css (if it isn't there yet)
                            string customcsscontents = File.ReadAllText(customcsspath);
                            string fadelcss = GetResourceStreamAsString("Adam.Fadel.CommandLine.Custom.css");
                            if (!customcsscontents.Contains(fadelcss))
                            {
                                customcsscontents += fadelcss;
                                File.WriteAllText(customcsspath, customcsscontents);
                            }
                        }
                    }

                    //Creating and updating settings
                    Guid categoryId = EnsureSettingCategoryExists("Fadel ARC");
                    EnsureSettingDefinitionExists("Fadel_EndpointUri",
                        "Fadel Endpoint URL", categoryId,
                        @"http://yourserver:8082/ipmSuite5/ws/V1");
                    EnsureSettingDefinitionExists("Fadel_EndpointUsername",
                        "Fadel Endpoint Username", categoryId,
                        "");
                    EnsureSettingDefinitionExists("Fadel_EndpointPassword",
                        "Fadel Endpoint Password", categoryId, "");
                    ReplaceProvider(".assetStudioSearchRecordDisplays",
                        "Adam.Web.AssetStudio.Displays.RecordMosaicDisplay, Adam.Web.AssetStudio",
                        "Adam.Fadel.Web.MosaicDisplays.RecordMosaicDisplay, Adam.Fadel.Web");
                    ReplaceProvider(".assetStudioViewBasketRecordDisplays",
                        "Adam.Web.AssetStudio.Displays.RecordMosaicDisplay, Adam.Web.AssetStudio",
                        "Adam.Fadel.Web.MosaicDisplays.RecordMosaicDisplay, Adam.Fadel.Web");
                    const string action = @"Adam.Fadel.Web.Actions.AssetRightsClearanceAction, Adam.Fadel.Web";
                    EnsureSettingContainsProvider(".assetStudioSearchBasketActions", action);
                    EnsureSettingContainsProvider(".assetStudioSearchRecordActions", action);
                    EnsureSettingContainsProvider(".assetStudioSelectionActions", action);
                    EnsureSettingContainsProvider(".assetStudioViewBasketActions", action);
                    EnsureSettingContainsProvider(".assetStudioViewBasketRecordActions", action);
                    EnsureSettingContainsProvider(".assetStudioViewRecordActions", action);
                    _app.LogOff();
                }
            }
        }

        private static void ExtractResource(string resourceName, string fileName)
        {
            using (Stream resource = Assembly.GetExecutingAssembly().GetManifestResourceStream(resourceName))
            {
                using (var file = new FileStream(fileName, FileMode.Create, FileAccess.Write))
                {
                    resource.CopyTo(file);
                }
            }
        }

        private static string GetResourceStreamAsString(string resourceName)
        {
            using (Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(resourceName))
            {
                using (var reader = new StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        private static void EnsureSettingContainsProvider(string settingName, string provider)
        {
            var setting = new XmlSetting(_app);
            setting.LoadSystemValue(settingName);

            var xmlDocument = new XmlDocument();
            xmlDocument.LoadXml(setting.Value);

            //Check if provider is already registered
            XmlNodeList nodes = xmlDocument.SelectNodes(
                string.Format("//add[@type='{0}']", provider));
            if (nodes.Count == 0)
            {
                //If it's not registered; just add it to the end of the list
                nodes = xmlDocument.SelectNodes("//add");
                XmlNode node = nodes[nodes.Count - 1].Clone();
                node.Attributes["type"].Value = provider;
                nodes[nodes.Count - 1].ParentNode.AppendChild(node);
                setting.Value = xmlDocument.OuterXml;
                setting.Save();
            }

            setting.Save();
        }

        private static void ReplaceProvider(string settingName, string oldProvider, string newProvider)
        {
            var setting = new XmlSetting(_app);
            setting.LoadSystemValue(settingName);
            setting.Value = setting.Value.Replace(oldProvider, newProvider);
            setting.Save();
        }

        private static Guid EnsureSettingDefinitionExists(string name, string label, Guid categoryId,
            string defaultValue)
        {
            var settingDefinition = new TextSettingDefinition(_app);
            if (settingDefinition.TryLoad(name) == TryLoadResult.NotFound)
            {
                settingDefinition.AddNew();
                settingDefinition.CategoryId = categoryId;
                settingDefinition.Name = name;
            }
            settingDefinition.Labels[Language.EnglishId].Value = label;
            settingDefinition.DefaultValue = defaultValue;
            settingDefinition.AllowSystemSetting = true;
            settingDefinition.Save();
            return settingDefinition.Id;
        }

        private static Guid EnsureSettingCategoryExists(string name)
        {
            var category = new SettingCategory(_app);
            if (category.TryLoad(name) == TryLoadResult.NotFound)
            {
                category.AddNew();
                category.Name = name;
            }
            category.Save();
            return category.Id;
        }

        private static void ShowInstallHelp(string command, Arguments args)
        {
        }

        private static void DisplayHelp(Arguments args, string[] commands)
        {
            Console.WriteLine(
                "Syntax:\n" +
                "Adam.Fadel.CommandLine Command [-option1=Value] [-option2=Value]\n\n" +
                "The Command argument is always required, and can be followed by some optional\n" +
                "or required options. The valid commands are:\n\n\t" +
                String.Join("\n\t", commands) + "\n\n" +
                "To get more information about any of these commands, use the -? argument.\n" +
                "For example:\n" +
                "\tAdam.Workflow.CommandLine " + commands[0] + " -?\n");
        }
    }
}