﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using Adam.Core;
using Adam.Fadel.Web.Actions;
using Adam.Fadel.Web.AssetAnswerJsonTypes;
using Attribute = Adam.Fadel.Web.RightsAttributesJsonTypes.Attribute;
using RootObject = Adam.Fadel.Web.RightsAttributesJsonTypes.RootObject;
using Adam.Tools.ExceptionHandler;
using Newtonsoft.Json;

namespace Adam.Fadel.Web
{
    public class FadelHelper
    {
        private readonly string _BaseUri;
        private readonly string _Password;
        private readonly string _UserName;

        public FadelHelper(Application app)
        {
            _BaseUri = app.GetSetting("Fadel_EndpointUri").ToString();
            _UserName = app.GetSetting("Fadel_EndpointUsername").ToString();
            _Password = app.GetSetting("Fadel_EndpointPassword").ToString();
        }

        public Dictionary<int, string> GetFormats()
        {
            return GetRightAttributes(UrlCombine(_BaseUri, @"/rightWS/rightAttributes/20"));
        }

        public Dictionary<int, string> GetTerritories()
        {
            return GetRightAttributes(UrlCombine(_BaseUri, @"/rightWS/rightAttributes/30"));
        }

        private Dictionary<int, string> GetRightAttributes(string uri)
        {
            string json = GetJson(uri, "", "application/x-www-form-urlencoded");

            var attributes = JsonConvert.DeserializeObject<RootObject>(json);

            var rights = new Dictionary<int, string>();
            if ((attributes != null) && (attributes.attribute.Count > 0))
            {
                ProcessRightAttributes(attributes.attribute, "", rights);
            }
            return rights;
        }

        private string GetJson(string url, string data, string contentType)
        {
            var request = (HttpWebRequest) WebRequest.Create(url);
            byte[] encodedData = Encoding.ASCII.GetBytes(data);
            SetBasicAuthHeader(request, _UserName, _Password);
            request.Method = "POST";
            request.ContentType = contentType;
            request.Accept = "application/json";
            request.ContentLength = encodedData.Length;

            using (Stream stream = request.GetRequestStream())
            {
                stream.Write(encodedData, 0, encodedData.Length);
            }

            try
            {
                var ws = (HttpWebResponse) request.GetResponse();
                if (ws.StatusCode != HttpStatusCode.OK)
                {
                    throw ExceptionManager.CreateInvalidOperationException(
                        string.Format("Fadel ARC: {0}", ws.Headers["result"]), false);
                }
                Stream dataStream = ws.GetResponseStream();
                var reader = new StreamReader(dataStream);
                return reader.ReadToEnd();
            }
            catch (WebException e)
            {
                throw ExceptionManager.CreateInvalidOperationException(
                    string.Format("Fadel ARC: {0}", e.Message), false);
            }
        }

        private void ProcessRightAttributes(List<Attribute> list, string parentPath, Dictionary<int, string> result)
        {
            foreach (Attribute attribute in list)
            {
                string currentPath = FormatPath(parentPath, attribute.right.description);
                result.Add(attribute.rightRelationshipId, currentPath);
                ProcessRightAttributes(attribute.childrenLst, currentPath, result);
            }
        }

        private string FormatPath(string path1, string path2)
        {
            if (path1.Length == 0)
            {
                return path2;
            }
            return string.Format("{0} > {1}", path1, path2);
        }

        public void SetBasicAuthHeader(WebRequest req, String userName, String userPassword)
        {
            string authInfo = userName + ":" + userPassword;
            authInfo = Convert.ToBase64String(Encoding.Default.GetBytes(authInfo));
            req.Headers["Authorization"] = "Basic " + authInfo;
        }


        internal Dictionary<string, ClearanceResultInfo> GetClearanceResult(string[] assetIds,
            string format,
            string territory,
            DateTime? inDateFromDate,
            DateTime? inDateToDate,
            DateTime? outDateFromDate,
            DateTime? outDateToDate)
        {
            //Build JSON request body
            string jsonRequestBody = GetJsonRequestBody(assetIds,
                format,
                territory,
                inDateFromDate,
                inDateToDate,
                outDateFromDate,
                outDateToDate);

            string json =
                GetJson(
                    UrlCombine(_BaseUri, @"/clearanceWS/assetClearance"),
                    jsonRequestBody,
                    "application/json");

            var response = JsonConvert.DeserializeObject<AssetClearanceJsonTypes.RootObject>(json);

            var result = new Dictionary<string, ClearanceResultInfo>();
            for (int i = 0; i < response.clearance.Count; i++)
            {
                string assetId = response.clearance[i].asset.assetExtId;

                if (result.ContainsKey(assetId))
                {
                    throw ExceptionManager.CreateInvalidOperationException(
                        string.Format("Fadel ARC returned more than one asset with id {0}", assetId), false);
                }

                ClearanceResultStatus status;
                string additionalInfo = "";
                switch (response.clearance[i].typeName)
                {
                    case "NOT AVAILABLE":
                        status = ClearanceResultStatus.NotAvailable;
                        break;
                    case "AVAILABLE":
                        status = ClearanceResultStatus.Available;
                        break;
                    case "AVAILABLE WITH EXCEPTIONS":
                        status = ClearanceResultStatus.AvailableWithExceptions;
                        additionalInfo = GetClearanceAdditionalInfo(response.clearance[i].notAvailableAnswerListUri,
                            GetJsonRequestBody(new[] {assetId},
                                format,
                                territory,
                                inDateFromDate,
                                inDateToDate,
                                outDateFromDate,
                                outDateToDate));
                        break;
                    default:
                        status = ClearanceResultStatus.Error;
                        break;
                }
                result.Add(assetId, new ClearanceResultInfo
                {
                    AssetId = assetId,
                    Status = status,
                    AvailableCount = response.clearance[i].availableCount,
                    AvailableExceptCount = response.clearance[i].availableExceptCount,
                    AdditionalInfo = additionalInfo
                });
            }

            return result;
        }

        private string GetClearanceAdditionalInfo(string url, string body)
        {
            string json =
                GetJson(
                    url,
                    body,
                    "application/json");

            var response = JsonConvert.DeserializeObject<AssetAnswerJsonTypes.RootObject>(json);

            var result = new List<string>();
            foreach (AssetAnswer item in response.assetAnswer)
            {
                result.Add(string.Format("{0}/{1}", item.answer.format.right.description,
                    item.answer.territory.right.description));
            }

            if (result.Count > 0)
            {
                return string.Format("Not available for {0}", string.Join(", ", result.ToArray()));
            }

            return "";
        }

        private string GetJsonRequestBody(string[] assetIds, string format, string territory, DateTime? inDateFromDate,
            DateTime? inDateToDate, DateTime? outDateFromDate, DateTime? outDateToDate)
        {
            var jsonRequestBody = new StringBuilder();
            jsonRequestBody.AppendFormat(@"{{");
            if (inDateToDate.HasValue)
            {
                jsonRequestBody.AppendFormat(@"   ""inDateTo"": {0},", ToJavaDate(inDateToDate.Value));
            }
            if (outDateToDate.HasValue)
            {
                jsonRequestBody.AppendFormat(@"   ""outDateTo"": {0},", ToJavaDate(outDateToDate.Value));
            }
            jsonRequestBody.AppendFormat(@"   ""rights"": {{");
            jsonRequestBody.AppendFormat(@"       ""selected"": {{");
            jsonRequestBody.AppendFormat(@"           ""20"": {{");
            if (format.Length > 0)
            {
                jsonRequestBody.AppendFormat(@"               ""{0}"": {{}}", format);
            }
            jsonRequestBody.AppendFormat(@"           }},");
            jsonRequestBody.AppendFormat(@"           ""30"": {{");
            if (territory.Length > 0)
            {
                jsonRequestBody.AppendFormat(@"               ""{0}"": {{}}", territory);
            }
            jsonRequestBody.AppendFormat(@"           }}");
            jsonRequestBody.AppendFormat(@"       }}");
            jsonRequestBody.AppendFormat(@"   }},");
            if (inDateFromDate.HasValue)
            {
                jsonRequestBody.AppendFormat(@"   ""inDateFrom"": {0},", ToJavaDate(inDateFromDate.Value));
            }
            if (outDateFromDate.HasValue)
            {
                jsonRequestBody.AppendFormat(@"   ""outDateFrom"": {0},", ToJavaDate(outDateFromDate.Value));
            }
            jsonRequestBody.AppendFormat(@"   ""externalAssets"": {{");
            jsonRequestBody.AppendFormat(@"       ""selected"": {{");
            var jsonExternalIdentifier = new List<string>();
            for (int i = 0; i < assetIds.Length; i++)
            {
                jsonExternalIdentifier.Add(string.Format(@" ""{0}"": {{ ""externalIdentifier"": ""{0}"" }}", assetIds[i]));
            }
            jsonRequestBody.Append(string.Join(",", jsonExternalIdentifier.ToArray()));
            jsonRequestBody.AppendFormat(@"       }}");
            jsonRequestBody.AppendFormat(@"   }}");
            jsonRequestBody.AppendFormat(@"}}");

            return jsonRequestBody.ToString();
        }

        private long ToJavaDate(DateTime dateTime)
        {
            TimeSpan span = dateTime - new DateTime(1970, 1, 1);
            return (long) span.TotalMilliseconds;
        }

        private string UrlCombine(string baseUrl, string relativeUrl)
        {
            if (baseUrl.Length == 0)
                return relativeUrl;
            if (relativeUrl.Length == 0)
                return baseUrl;
            return string.Format("{0}/{1}", baseUrl.TrimEnd(new[] {'/', '\\'}), relativeUrl.TrimStart(new[] {'/', '\\'}));
        }
    }
}